#ifndef CANINTERFACE_H
#define CANINTERFACE_H

    // Message return codes ------------------------------------------------------
    const int RX_Message = 0;
    const int RX_NoMessage = 1;
    const int RX_FaultyMessage = 2;
    const int RX_HEART = 3;
    const int RX_ACK_FA = 4;
    const int RX_ACK_FD = 5;
    const int RX_ACK_TX = 6;
    const int RX_ACK_RESET = 7;
    const int RX_ACK_SETPARAM = 8;
    const int RX_ACK_SETPARAM1 = 9;
    const int RX_RS = 10;
    const int RX_ACK_MSGMODE = 11;
    const int RX_ACK_TXL = 12;
    const int RX_VERSION = 13;
    const int RX_ACK_SETACK = 14;
    const int RX_ACK_SETHEART = 15;
    const int RX_ACK_FLASH = 16;
    const int RX_TSMessage = 17;
    const int RX_ACK_FAX = 18;

    // Message Format ------------------------------------------------------------
    const byte MSG_TOKEN_START = 192;
    const byte MSG_TOKEN_ESC = 219;
    const byte MSG_START_STUFF = 220;
    const byte MSG_ESC_STUFF = 221;

    const byte MSG_OVERHEAD = 3;
    const byte TOTAL_OVERHEAD = 8;
    const byte RXTX_TOTALOVERHEAD = 11;

    const byte MIN_MSG_LEN = 6;

    // Command message IDs
    const byte MSG_ID_ACK = 0;
    const byte MSG_ID_FA = 1;
    const byte MSG_ID_FD = 2;

    const byte MSG_ID_TX = 3;
    const byte MSG_ID_RX = 4;
    const byte MSG_ID_RESET = 5;
    const byte MSG_ID_HEART = 6;
    const byte MSG_ID_SETPARAM = 7;
    const byte MSG_ID_RQ = 8;
    const byte MSG_ID_RS = 9;
    const byte MSG_ID_FLASH = 10;
    const byte MSG_ID_SETACK = 11;
    const byte MSG_ID_SETHEART = 12;
    const byte MSG_ID_VERSION = 13;
    const byte MSG_ID_SETPARAM1 = 14;
    const byte MSG_ID_SETMSGMODE = 15;
    const byte MSG_ID_TXL = 16;
    const byte MSG_ID_TXP = 17;
    const byte MSG_ID_TXPL = 18;
    const byte MSG_ID_RXDATAX = 19;
    const byte MSG_ID_SETRTC = 20;
    const byte MSG_ID_GETRTC = 21;
    const byte MSG_ID_FAX = 22;

    // RESET unlock key
    const byte RESET_UNLOCK_KEY1 = 0xA5;
    const byte RESET_UNLOCK_KEY2 = 0x69;
    const byte RESET_UNLOCK_KEY3 = 0x5A;
    const byte RESET_UNLOCK_KEY4 = 0x5B;

    // Message lengths without overhead (ID + MSGLEN) incl. checksum
    const int MSG_LEN_MSB = 0;
    const int MSG_LEN_ACK = 3;
    const int MSG_LEN_FA = 5;
    const int MSG_LEN_FD = 5;
    const int MSG_LEN_FX = 7;
    const int MSG_LEN_TX = 16;
    const int MSG_LEN_RX = 16;
    const int MSG_LEN_RESET = 5;
    const int MSG_LEN_STATS = 16;
    const int MSG_LEN_SET = 14;
    const int MSG_LEN_RQ = 3;
    const int MSG_LEN_RS = 4;
    const int MSG_LEN_MM = 3;

    // Message field indexes
    const int MSG_IDX_ID = 3;
    const int MSG_IDX_ACK_MSG = 4;

    const int FAFD_IDX_PGNMSB = 4;
    const int FAFD_IDX_PGN2ND = 5;
    const int FAFD_IDX_PGNLSB = 6;
    const int FAFD_IDX_CHKSUM = 7;
    const int FAX_IDX_CHKSUM = 9;

    const int RXTX_IDX_MSGSTART = 0;
    const int RXTX_IDX_MSGLENMSB = 1;
    const int RXTX_IDX_MSGLENLSB = 2;
    const int RXTX_IDX_MSGID = 3;
    const int RXTX_IDX_PGNMSB = 4;
    const int RXTX_IDX_PGN2ND = 5;
    const int RXTX_IDX_PGNLSB = 6;
    const int RXTX_IDX_DESTADDR = 7;
    const int RXTX_IDX_SRCADDR = 8;
    const int RXTX_IDX_PRIORITY = 9;
    const int RXTX_IDX_DATASTART = 10;

    const int RESET_IDX_ULKEY1 = 4;
    const int RESET_IDX_ULKEY2 = 5;
    const int RESET_IDX_ULKEY3 = 6;

    const int SET_IDX_NAME = 4;
    const int SET_IDX_SA = 12;
    const int SET_IDX_ADDRRANGE_BOTTOM = 13;
    const int SET_IDX_ADDRRANGE_TOP = 14;
    const int SET_IDX_OPMODE = 15;

    const int RQ_IDX_REQUESTED = 4;

    const int RS_IDX_STATUS = 4;
    const int RS_IDX_SA = 5;

    // Filters -------------------------------------------------------------------
    struct MsgFilter
    {
        long lMsgID;
        bool bMsgActive;
    };

    const int MAXFILTER = 100;
    const int MINSAMPLERATE = 100;

    // ---------------------------------------------------------------------------
    // CAN Message Format
    // ---------------------------------------------------------------------------
    struct Msg
    {
        byte Data[REC_BUFFER_SIZE]; // The maximum J1939 message length = 255 x 7 = 1785
                                    // This buffer size provides sufficient space for stuff bytes
        int DataLen;
    };

    class CANInterface
    {
        public:
            CANInterface();
            virtual ~CANInterface();

            int Initialize(int* fileCOMPort, char* sPort, bool* bCommunicationError, int* nErrNum,
                           bool* bErrorOpenCOMPort, bool* bErrorCOMPortSettings);
            void InitializeBuffers(void);
            void ClosePort(int fileCOMPort);
            int Transmit(Msg frame, int fileCOMPort);
            int Transmit(byte* Data, int DataLen, int fileCOMPort);
            int Receive(Msg* frame, int fileCOMPort);
            int FilterSystemMessage(Msg*);
            void CleanReceiveBuffer(int, int);
            void RemoveStuffBytes(void);
            void ShiftCopyBuffer(int);
            byte ComputeCheckSum(byte*);
            int CreateFilterMessage(byte* pMsg, long lPGN, bool bAdd);
            int SendFilterMessage(long lPGN, int fileCOMPort, bool bAdd);
            int SendFilterMessageX(long lPGN, int nSampleRate, int fileCOMPort);
            int CreateFilterMessageX(byte* pMsg, long lPGN, int nSampleRate);
            int SendSETMessage(byte* nNAME, byte nSA, byte nSABottom, byte nSATop, byte nOpMode, bool bReqRSMessage, int fileCOMPort);
            int SendRESETMessage(int fileCOMPort, int nCANBaudRate);
            int SendRequestMessage(int fileCOMPort, byte cRequestID);
            int SendMessageMode(int fileCOMPort, byte nMsgMode);

            // Misc. variables
            bool bHeartbeat;
            char sHWVersion[8];
            char sSWVersion[8];
            int nErrCntChecksum;
            int nErrCntStuffing;

            int nReportedAddrClaimStatus;
            int nReportedSourceAddress;

        protected:

        private:

    };

#endif // CANINTERFACE_H
