#ifndef CONFIG_H_INCLUDED
#define CONFIG_H_INCLUDED

// Configuration Settings
// -----------------------------------------------------------------------------
const int OK = 0;
const int ERROR = 1;

#define byte                unsigned char

// Serial Interface
// Use "dmesg | grep tty" to determine active USB port
//#define COMPORT             "/dev/ttyUSB0"
#define COMPORT             "/dev/ttyACM0"

// File name
#define FILENAME			"jCOMRecord.csv"

// CAN Baudrates
const byte CANBAUDRATE_250 = 0;
const byte CANBAUDRATE_500 = 1;
const byte CANMSGLEN = 8;

const int REC_BUFFER_SIZE = 2048;       // Applies to the receive buffer
                                        // The maximum J1939 message length = 255 x 7 = 1785
                                        // This buffer size provides sufficient space for stuff bytes
const int COM_BUFFER_SIZE = 256;
const int READ_TIMEOUT = 500;
const int WRITE_TIMEOUT = 500;
const int REC_BUFFER_FILLTIME = 80;

// Gateway operation mode
const byte OPMODE_LISTENONLY = 0;
const byte OPMODE_EVENT = 1;

const byte MSGMODE_ECU = 0;
const byte MSGMODE_GATEWAY1 = 1;
const byte MSGMODE_GATEWAY2 = 2;

// J1939 Message Buffers
const int MAXDATALEN = 1785;
const int APPMSGBUFFERSIZE = 20;            // Size of application message buffer

// J1939 Message Filter
const long PGN_ALLOW_ALL = 0x100000;
const long PGN_SAMPLE_RX = 65288;           // 0xFF08

// J1939 Transmit
const long PGN_SAMPLE_TX = 65281;
const int DEST_ADDR_SAMPLE = 128;

// J1939 NAME Default settings
const long NAME_IDENTITY_NUMBER = 0x1FFFFF;         // 21 bits - Model No.
const long NAME_MANUFACTURER_CODE = 0x7FF;          // 11 bits
const long NAME_FUNCTION_INSTANCE = 1;              // 5 bits
const long NAME_ECU_INSTANCE = 1;                   // 3 bits  - NVR Instance
const long NAME_FUNCTION = 0xFF;                    // 8 bits
const long NAME_RESERVED = 0;                       // 1 bit
const long NAME_VEHICLE_SYSTEM = 0x7F;              // 7 bits
const long NAME_VEHICLE_SYSTEM_INSTANCE = 16;       // 4 bits  - Vehicle Instance
const long NAME_INDUSTRY_GROUP = 0x01;              // 3 bits
const long NAME_ARBITRARY_ADDRESS_CAPABLE = 0x01;   // 1 bit

// J1939 Node Address Settings
const byte SA_PREFERRED = 128;                      // Preferred node address
const byte ADDRESSRANGEBOTTOM = 128;                // Address range low
const byte ADDRESSRANGETOP = 247;                   // Address range high
const byte NULLADDRESS = 254;
const byte GLOBALADDRESS = 255;

// Address Claim Status as received from the jCOM gateway
const int RS_STATUS_NONE = 0;
const int RS_STATUS_ADDRESSCLAIMINPROGRESS = 1;
const int RS_STATUS_ADRESSCLAIMSUCCESSFUL = 2;
const int RS_STATUS_ADDRESSCLAIMFAILED = 3;
const int RS_STATUS_LISTENONLYMODE = 4;

#endif // CONFIG_H_INCLUDED
