#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "config.h"
#include "CANInterface.h"
#include "COM1939.h"

using namespace std;

// Reference to the COM1939 class
static COM1939 com1939;

byte msgNAME[] =
{
  (byte)(NAME_IDENTITY_NUMBER & 0xFF),
  (byte)((NAME_IDENTITY_NUMBER >> 8) & 0xFF),
  (byte)(((NAME_MANUFACTURER_CODE << 5) & 0xFF) | (NAME_IDENTITY_NUMBER >> 16)),
  (byte)(NAME_MANUFACTURER_CODE >> 3),
  (byte)((NAME_FUNCTION_INSTANCE << 3) | NAME_ECU_INSTANCE),
  (byte)(NAME_FUNCTION),
  (byte)(NAME_VEHICLE_SYSTEM << 1),
  (byte)((NAME_ARBITRARY_ADDRESS_CAPABLE << 7) | (NAME_INDUSTRY_GROUP << 4) | (NAME_VEHICLE_SYSTEM_INSTANCE))
};

byte SampleTxData[] = {0x38, 0x37, 0x36, 0x35, 0x34, 0x33, 0x32, 0x31};

// Function calls in this module
int kbhit(char* key);
int kbNumber(long *lNumber, int nDigits);

int main()
{
    // Declarations
    char sCOMPort[20];
    int nIndex;

    long lPGN;
    int nPriority;
    int nSourceAddress;
    int nDestAddress;
    int nDataLen;
    byte nData[MAXDATALEN];

    bool bTimeStamp = false;
    byte nTimeStamp[8];

    int nFile;
    bool bWriteToFile = false;
    char strToFile[5000];   // Must support TP length
    char strData[50];
    int msec = 0;

    // Initialize the port and protocol settings
    strcpy(sCOMPort, COMPORT);
    com1939.Initialize(&sCOMPort[0]);
    com1939.SendMessageMode(MSGMODE_GATEWAY2);

    if(com1939.bCommunicationError == false)
    {
        printf("COM Port Init OK. %s\n\r", com1939.sPort);
        printf("Press 'h' for Help\n\r");
    }// end if
    else if(com1939.bErrorOpenCOMPort == true)
        printf("Unable to open COM port: %s - ErrNo: %d\n\r", com1939.sPort, com1939.nErrNum);
    else if(com1939.bErrorCOMPortSettings == true)
        printf("Unable to apply COM port settings: %s - ErrNo: %d\n\r", com1939.sPort, com1939.nErrNum);

    // Check if recording file exists
    nFile = open(FILENAME, O_WRONLY | O_CREAT | O_EXCL, 0644);
    if(nFile < 0)
        std::remove(FILENAME);

    // Main loop
    bool bExit = false;
    bool bAllowAllPGNs = false;
    bool bHeartbeat = false;
    while(1)
    {
        // Wait for assigned system loop time period
        usleep(SYSTEM_LOOP_TIME);

        // Call the COM1939 communication protocol
        int nStatus = com1939.Receive(&lPGN, &nPriority, &nSourceAddress, &nDestAddress, &nData[0], &nDataLen,
                                      &bTimeStamp, &nTimeStamp[0]);

        switch(nStatus)
        {
            case RX_Message:
            case RX_TSMessage:        // PGN received

                // Check whether the data goes to the screen or to file
                if(bWriteToFile == false)
                {
                    // Print to screen
                    printf("PGN:%ld P:%d SA:%d DA:%d Len:%d Data:", lPGN, nPriority, nSourceAddress, nDestAddress, nDataLen);
                    for(nIndex = 0; nIndex < nDataLen; nIndex++)
                        printf("%2x ", nData[nIndex]);

                   // Check for timestamp information
                    if(bTimeStamp == true)
                    {
                        msec = ((int)nTimeStamp[6] << 8) + (int)nTimeStamp[7];
                        printf("%d-%02d-%02d %02d:%02d:%02d.%03d", 2000 + (int)nTimeStamp[0], nTimeStamp[1], nTimeStamp[2],
                                                               nTimeStamp[3], nTimeStamp[4], nTimeStamp[5], msec);

                    }// end if

                    printf("\n\r");

                }// end if
                else
                {
                    // Print to file
                    sprintf(strToFile, "%ld,%d,%d,%d,%d,", lPGN, nPriority, nSourceAddress, nDestAddress, nDataLen);

                    // Determine between regular PGN and TP
                    if(nDataLen <= CANMSGLEN)
                    {
                        for(nIndex = 0; nIndex < CANMSGLEN; nIndex++)
                        {
                            if(nIndex < nDataLen)
                            {
                                sprintf(strData,"%2x", nData[nIndex]);
                                strcat(strToFile, strData);

                            }// end if
                            else
                                strcat(strToFile, "00");

                        }// end for

                    }// end if
                    else
                    {
                        for(nIndex = 0; nIndex < nDataLen; nIndex++)
                        {
                            sprintf(strData, "%2x", nData[nIndex]);
                            strcat(strToFile, strData);

                        }// end for

                    }// end else

                   // Check for timestamp information
                    if(bTimeStamp == true)
                    {
                        msec = ((int)nTimeStamp[6] << 8) + (int)nTimeStamp[7];
                        sprintf(strData, ",%d-%02d-%02d,%02d:%02d:%02d.%03d", 2000 + (int)nTimeStamp[0], nTimeStamp[1], nTimeStamp[2],
                                                               nTimeStamp[3], nTimeStamp[4], nTimeStamp[5], msec);
                        strcat(strToFile, strData);

                    }// end if

                    strcat(strToFile, "\n");

                    // Write to file
                    write(nFile, &strToFile[0], (ssize_t)strlen(strToFile));
                    printf("%s", strToFile);

                }// end else

                break;

            case RX_ACK_FA:
                printf(" FA ACK\n\r");
                break;

            case RX_ACK_FAX:
                printf(" FAX ACK\n\r");
                break;

            case RX_ACK_FD:
                printf(" FD ACK\n\r");
                break;

            case RX_ACK_SETPARAM:
                printf(" SETPARAM ACK\n\r");
                break;

            case RX_ACK_SETPARAM1:
                printf(" SETPARAM1 ACK\n\r");
                break;

            case RX_ACK_RESET:
                printf(" RESET ACK\n\r");
                break;

            case RX_ACK_MSGMODE:
                printf(" SETMSGMODE ACK\n\r");
                break;

            case RX_HEART:
                if(bHeartbeat == true)
                {
                    if(com1939.bjCOM1939Heartbeat == true)
                        printf("Tick\n\r");
                    else
                        printf("Tock\n\r");
                }
                break;

            case RX_RS:
                int nSA;
                int nJ1939Status;
                nJ1939Status = com1939.GetStatus(&nSA);
                printf(" Status: \n\r");
                printf("SA: %d - ", nSA );

                switch(nJ1939Status)
                {
                    case RS_STATUS_NONE:
                        printf("No J1939 Address Claimed.\n\r");
                        break;

                    case RS_STATUS_ADDRESSCLAIMINPROGRESS:
                        printf("Address Claim in Progress.\n\r");
                        break;

                    case RS_STATUS_ADRESSCLAIMSUCCESSFUL:
                        printf("Address Claim Successful.\n\r");
                        break;

                    case RS_STATUS_ADDRESSCLAIMFAILED:
                        printf("Address Claim Failed.\n\r");
                        break;

                    case RS_STATUS_LISTENONLYMODE:
                        printf("Listen-Only Mode.\n\r");
                        break;

                }// end switch

                break;

        }// end switch

        // Check for user interaction
        char key;
        if(kbhit(&key))
        {
            switch(key)
            {
                case '1':	// Reset gateway; initialize CAN at 250 kbps
                    com1939.ResetGateway(CANBAUDRATE_250);
                    com1939.SendMessageMode(MSGMODE_GATEWAY2);
                    break;

                case '2':	// Reset gateway; initialize CAN at 500 kbps
                    com1939.ResetGateway(CANBAUDRATE_500);
                    com1939.SendMessageMode(MSGMODE_GATEWAY2);
                    break;

                case 'a':       // Allow all messages (Toggle)
                    bAllowAllPGNs = !bAllowAllPGNs;
                    if(bAllowAllPGNs == true)
                        com1939.AddFilter(PGN_ALLOW_ALL);
                    else
                        com1939.DelFilter(PGN_ALLOW_ALL);
                    break;

                case 'b':
                    bHeartbeat = !bHeartbeat;
                    break;

                case 'c':
                    com1939.SetProtocolParameters(msgNAME, SA_PREFERRED, ADDRESSRANGEBOTTOM, ADDRESSRANGETOP, OPMODE_EVENT, true);
                    break;

                case 'd':
                    com1939.DelFilter(PGN_SAMPLE_RX);
                    break;

                case 'f':
                    com1939.AddFilter(PGN_SAMPLE_RX);
                    break;

                case 'h':       // Help message
                    printf("\n\r1 - Reset gateway; Initialize CAN Bus at 250 kbps");
                    printf("\n\r2 - Reset gateway; Initialize CAN Bus at 500 kbps");
                    printf("\n\ra - Toggle Pass [a]ll PGNs");
                    printf("\n\rb - Toggle heart[b]eat");
                    printf("\n\rc - [c]laim address");
                    printf("\n\rd - [d]elete PGN filter");
                    printf("\n\rf - Add PGN [f]ilter");
                    printf("\n\rr - Add PGN filter with sample [r]ate");
                    printf("\n\rs - Request J1939 [s]tatus");
                    printf("\n\rt - [t]ransmit sample PGN");
                    printf("\n\rv - Get hardware & software [v]ersion");
                    printf("\n\rw - Toggle [w]rite to file");
                    printf("\n\rx - E[x]it\n\r");
                    break;

                case 'r':
                    com1939.AddFilterX(PGN_SAMPLE_RX, 1000);
                    break;

                case 's':
                    com1939.RequestStatus();
                    break;

                case 't':
                    int nSA;
                    int nJ1939Status;
                    nJ1939Status = com1939.GetStatus(&nSA);

                    if(nJ1939Status == RS_STATUS_ADRESSCLAIMSUCCESSFUL)
                    {
                        // Transmit the sample data
                        com1939.Transmit(6, PGN_SAMPLE_TX, nSA, DEST_ADDR_SAMPLE, &SampleTxData[0], 8);
                        printf("Transmitted PGN %ld to node #%d:\n\r", PGN_SAMPLE_TX, DEST_ADDR_SAMPLE);
                        for(int nIndex = 0; nIndex < 8; nIndex++)
                            printf("0x%x ", SampleTxData[nIndex]);
                        printf("\n\r");

                    }// end if
                    else
                    {
                        // Print an error message
                        printf("Unable to Transmit - Reason: ");

                        switch(nJ1939Status)
                        {
                            case RS_STATUS_NONE:
                                printf("No J1939 Address Claimed.\n\r");
                                break;

                            case RS_STATUS_ADDRESSCLAIMINPROGRESS:
                                printf("Address Claim in Progress.\n\r");
                                break;

                            case RS_STATUS_ADDRESSCLAIMFAILED:
                                printf("Address Claim Failed.\n\r");
                                break;

                            case RS_STATUS_LISTENONLYMODE:
                                printf("Listen-Only Mode.\n\r");
                                break;

                        }// end switch

                    }// end else

                    break;

                case 'v':       // Get version strings
                    char sSWVersion[10];
                    char sHWVersion[10];
                    com1939.GetVersion(&sSWVersion[0], &sHWVersion[0]);
                    printf(" SW: %s HW: %s\n\r", sSWVersion, sHWVersion);
                    break;

                case'w':
                    // Toggle the "write" flag
                    if(bWriteToFile == false)
                    {
                        bWriteToFile = true;

                        // Open the file for recording
                        nFile = open(FILENAME, O_WRONLY | O_CREAT, 0644);

                        printf("Write = on.\n\r");

                    }// end if
                    else
                    {
                        bWriteToFile = false;
                        close(nFile);
                        printf("Write = off.\n\r");

                    }// end else
                    break;

                case 'x':       // Exit the program
                    com1939.Terminate();
                    bExit = true;
                    break;

            }// end switch

        }// end if

        if(bExit == true)
            break;

    }// end while

    return 0;

}// end main

//-FUNCTION-----------------------------------------------------------------
// Routine     : kbhit
// Description : Checks for keyboard input and fetches the code
// Returncode  : OK/ERROR
// -------------------------------------------------------------------------
int kbhit(char* key)
{
    struct termios oldt;
    struct termios newt;
    int ch;
    int oldf;

    tcgetattr(STDIN_FILENO, &oldt);
    newt = oldt;
    newt.c_lflag &= ~(ICANON | ECHO);
    tcsetattr(STDIN_FILENO, TCSANOW, &newt);
    oldf = fcntl(STDIN_FILENO, F_GETFL, 0);
    fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK);

    ch = getchar();
    *key = (char)ch;

    tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
    fcntl(STDIN_FILENO, F_SETFL, oldf);

    if(ch != EOF)
    {
        //ungetc(ch, stdin);
        return ERROR;
    }

    return OK;

}// end kbhit

